/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#include "GPumpApp.h"
#include "GPumpSettings.h"
#include "GPumpSettingsData.h"

#define GPUMP_APP_GET_PRIVATE(obj)\
  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GPUMP_TYPE_APP, GPumpAppPrivate))

G_DEFINE_TYPE (GPumpApp, gpump_app, GTK_TYPE_APPLICATION)

struct GPumpAppPrivate {
  
  /* Widgets */
  GtkWidget * window;
  GtkWidget * header_bar;
  GtkWidget * layout; /* GtkBox */
  GtkWidget * scroll_view; /* GtkScrolledWindow */
  GtkWidget * post_list_box; /* GtkListBox */
  GtkWidget * btn_gear; /* GtkMenuButton */
  GtkWidget * btn_new_post;
  GtkWidget * btn_refresh;
  
  /* gear menu */
  GtkWidget * gear_menu; /* GtkMenu */
  
  /* List store */
  GtkListStore * post_list_store;
};

typedef enum {
  GPUPM_APP_MAIN_LIST_COL = 0,
  GPUMP_APP_MAIN_LIST_NUM_COL
} GPumpAppCols;



/* * Callback and private function declarations *******************************/
void gpump_app_class_init (GPumpAppClass * klass);
void gpump_app_activate (GApplication * application);
void gpump_app_init (GPumpApp * self);

void _prepere_window (GPumpApp * self);
void _prepere_gear_menu (GPumpApp * self);
void _prepere_new_post_button (GPumpApp * self);
void _prepere_refresh_button (GPumpApp * self);
void _prepere_app_menu (GPumpApp * self);

void _cb_show_settings (GSimpleAction * action, GVariant * parameter,
                        gpointer user_data);
void _cb_do_nothing (GSimpleAction * action, GVariant * parameter,
                        gpointer user_data);
void _cb_quit (GSimpleAction * action, GVariant * parameter,
                        gpointer user_data);

/* * Class functions **********************************************************/
GPumpApp * gpump_app_new (void) {
  GPumpApp * self = g_object_new (GPUMP_TYPE_APP,
                       "application-id", "org.gego.gpump",
                       "flags",G_APPLICATION_FLAGS_NONE,
                       NULL);
  return self;
}

void gpump_app_init (GPumpApp * self) {
  g_print ("init\n");
  self->priv = GPUMP_APP_GET_PRIVATE (self);
  
}

void gpump_app_class_init (GPumpAppClass * klass) {
  g_print("GPumpApp class init.\n");
  
  // GObject * object_class = G_OBJECT_CLASS (klass);
  GApplicationClass * application_class = G_APPLICATION_CLASS (klass);
  
  application_class->activate = gpump_app_activate;
  
  g_type_class_add_private (klass, sizeof(GPumpAppPrivate));
  
  g_print ("GPumpApp class init done.\n");
}


void gpump_app_activate (GApplication * application) {
  g_print ("GPumpApp activate.\n");
  
  /* Check for uniques of window */
  GList * w_list = gtk_application_get_windows (GTK_APPLICATION(application));
  if (w_list) {
    gtk_window_present (GTK_WINDOW (w_list->data));
    return;
  }
  
  g_list_free (w_list);
  
  /* Prepere stuff */
  _prepere_window (GPUMP_APP (application));
  _prepere_gear_menu (GPUMP_APP (application));
  _prepere_app_menu (GPUMP_APP (application));
  _prepere_new_post_button (GPUMP_APP (application));
  _prepere_refresh_button (GPUMP_APP (application));
  
  /* Show it! */
  gtk_widget_show_all (GTK_WIDGET(GPUMP_APP(application)->priv->window));
  
  g_print ("done GPumpApp activate.\n");
}

void _prepere_window (GPumpApp * self) {
  GPumpAppPrivate * priv = self->priv;
  GtkWidget * window = priv->window =
    gtk_application_window_new (GTK_APPLICATION (self));
  GtkWidget * header_bar = priv->header_bar = gtk_header_bar_new ();
  GtkWidget * layout = priv->layout =
    gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
  
  gtk_header_bar_set_title (GTK_HEADER_BAR (header_bar), "GPump");
  gtk_header_bar_set_subtitle (GTK_HEADER_BAR (header_bar),
                               _("A Pump.io Client"));
  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (header_bar), TRUE);
  
  GtkWidget * scroll = priv->post_list_box =
    gtk_scrolled_window_new (NULL, NULL);
  
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
    GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  
  GtkWidget * post_list_box = priv->post_list_box =
    gtk_list_box_new ();
  
  gtk_container_add (GTK_CONTAINER(scroll), post_list_box);
  
  gtk_window_set_titlebar (GTK_WINDOW (window), header_bar);
  gtk_window_set_default_size (GTK_WINDOW(window), 500,500);
  
  gtk_box_pack_start (GTK_BOX (layout), scroll, TRUE, TRUE, 1);
  
  gtk_container_add (GTK_CONTAINER (window), layout);
  
  /* Hide window instead of destroying it */
  g_signal_connect (G_OBJECT (window), "delete_event" ,
                  G_CALLBACK (gtk_widget_hide_on_delete), self);
}

void _prepere_app_menu (GPumpApp * self) {
  /* App menu actions */
  const GActionEntry _app_actions[] = {
    {"settings", _cb_show_settings},
    {"about", _cb_do_nothing},
    {"quit", _cb_quit}
  };
  
  /* Map actions to application ... I do not know... :-S */
  g_action_map_add_action_entries (G_ACTION_MAP (self), _app_actions,
    G_N_ELEMENTS(_app_actions), GTK_APPLICATION(self));
  
  GMenu * menu = g_menu_new ();
  
  g_menu_append (menu, _("Settings"), "app.settings");
  g_menu_append (menu, _("About"), "app.about");
  g_menu_append (menu, _("Quit"), "app.quit");
  
  gtk_application_set_app_menu (GTK_APPLICATION (self), G_MENU_MODEL (menu));
  g_object_unref (menu);
}

void _prepere_gear_menu (GPumpApp * self) {
  GPumpAppPrivate * priv = self->priv;
  
  /* Create menu and menu items. */
  priv->gear_menu = gtk_menu_new();
  GtkWidget * mi_profiles = gtk_menu_item_new_with_label (_("Profiles"));
  
  
  /* Add items to menu. */
  gtk_container_add (GTK_CONTAINER (priv->gear_menu), mi_profiles);
  
  /* I have no idea why we need to run these, it seems trange, this
   * should be handled by the caller of gtk_
   */
  gtk_widget_show (GTK_WIDGET(mi_profiles));
  
  /* Hook up signals. */
  /*
  g_signal_connect (G_OBJECT (mi_profiles), "activate",
                    NULL, self);
  */
  
  /* create and attatch the gear menu button */
  priv->btn_gear = gtk_menu_button_new ();
  gtk_menu_button_set_popup (GTK_MENU_BUTTON (priv->btn_gear),
    GTK_WIDGET(priv->gear_menu));
  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->header_bar), priv->btn_gear);
  
  /* Setting icon */
  GtkWidget * gear_icon = gtk_image_new_from_icon_name(
    "emblem-system-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
  gtk_button_set_image (GTK_BUTTON (priv->btn_gear), gear_icon);
  
  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->btn_gear), _("GPump gear menu"));
}

void _prepere_new_post_button (GPumpApp * self) {
  GPumpAppPrivate * priv = self->priv;
  
  GtkWidget * share_icon = gtk_image_new_from_icon_name ("text-editor-symbolic",
    GTK_ICON_SIZE_LARGE_TOOLBAR);
  priv->btn_new_post = gtk_button_new ();
  gtk_button_set_image (GTK_BUTTON (priv->btn_new_post), share_icon);
  
  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->header_bar),
    priv->btn_new_post);
  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->btn_new_post),
    _("Create new post"));
}

void _prepere_refresh_button (GPumpApp * self) {
  GPumpAppPrivate * priv = self->priv;
  GtkWidget * refresh_icon = gtk_image_new_from_icon_name
    ("emblem-synchronizing-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
  priv->btn_refresh = gtk_button_new ();
  gtk_button_set_image (GTK_BUTTON (priv->btn_refresh), refresh_icon);
  gtk_header_bar_pack_start (GTK_HEADER_BAR(priv->header_bar),
    priv->btn_refresh);
  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->btn_refresh),
    _("Refresh the stream"));
}

/* * Callback implemenations **************************************************/
void _cb_show_settings (GSimpleAction * action, GVariant * parameter,
                        gpointer user_data) {
  GPumpApp * self = GPUMP_APP (user_data);
  GPumpSettings * settings = gpump_settings_new (self->priv->window);
  gpump_settings_show (settings);
}

void _cb_do_nothing (GSimpleAction * action, GVariant * parameter,
                        gpointer user_data) {}

void _cb_quit (GSimpleAction * action, GVariant * parameter,
                        gpointer user_data) {
  g_application_quit (G_APPLICATION(user_data));
}

